<?php

namespace Drupal\menu_block_settings_override\Traits;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\menu_block_settings_override\Plugin\Field\FieldType\MenuBlockSettingsOverrideItem;

/**
 * Trait MenuBlockSettingsOverrideTrait.
 *
 * @package Drupal\menu_block_settings_override\Traits
 */
trait MenuBlockSettingsOverrideTrait {

  /**
   * {@inheritdoc}
   */
  public function getPluginDefinition() {
    $definition = parent::getPluginDefinition();
    // override the provider so the module is not added to dependencies and config entities that use this plugin can be taken over by the menu_block module on uninstall.
    $definition['provider'] = 'menu_block';
    return $definition;
  }

  /**
   * @return bool|MenuBlockSettingsOverrideItem
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  protected function getOverriddenSettings() {
    $context_values = $this->getContextValues();
    if (!array_key_exists('node', $context_values) || empty($context_values['node'])) {
      return FALSE;
    }
    /** @var \Drupal\node\Entity\Node $node */
    $node = $this->getContextValue('node');
    $menu_block_fields = array_values(array_filter($node->getFields(), function (FieldItemListInterface $field) {
      return $field->getItemDefinition()
          ->getFieldDefinition()
          ->getType() == 'menu_block_settings_override';
    }));
    if (empty($menu_block_fields)) {
      return FALSE;
    }
    // Currently we only allow one field per entity type - so it's safe to take the first result.
    /** @var \Drupal\menu_block_settings_override\Plugin\Field\MenuBlockSettingsOverrideFieldItemList $field_value */
    $field_value = $menu_block_fields[0];
    /** @var \Drupal\block\Entity\Block[] $items */
    $items = $this->entityTypeManager->getStorage('block')->loadMultiple();
    // This is not a great way of getting the right instance but we join based on the label overridden in the configuration.
    $results = array_filter($items, function ($item) {
      /** @var \Drupal\block\Entity\Block $item */
      return $item->label() == $this->configuration['label'] && $item->getPluginId() == $this->getPluginId();
    });

    if (empty($results)) {
      return FALSE;
    }
    /** @var \Drupal\block\Entity\Block $block */
    $block = reset($results);
    $id = $block->id();
    return $field_value->getValueForMenuBlockId($id);
  }

  /**
   * {@inheritdoc}
   */
  public function build($menu_name = '') {
    $overridden_settings = $this->getOverriddenSettings();
    if ($overridden_settings) {
      $hidden = $overridden_settings->get('hidden')->getCastedValue();
      $status = $overridden_settings->get('status')->getCastedValue();
      if ($hidden) {
        return ['#access' => FALSE];
      }
      if ($status) {
        foreach ($overridden_settings->getOverridableFields() as $key) {
          $this->overrideIfApplicable($key, $overridden_settings, $this->configuration);
        }
      }
    }
    // Cached on a per-node level thanks to the node context definition: https://drupal.stackexchange.com/a/199601
    return parent::build();
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    $config = parent::getConfiguration();
    $overridden_settings = $this->getOverriddenSettings();
    if ($overridden_settings) {
      $hidden = $overridden_settings->get('hidden')->getCastedValue();
      $status = $overridden_settings->get('status')->getCastedValue();

      if ($hidden) {
        return $config;
      }
      if ($status) {
        foreach ($overridden_settings->getOverridableFields() as $key) {
          $this->overrideIfApplicable($key, $overridden_settings, $config);
        }
      }
    }
    return $config;
  }

  /**
   * @param string $key
   * @param \Drupal\menu_block_settings_override\Plugin\Field\FieldType\MenuBlockSettingsOverrideItem $overridden_settings
   * @param array $configuration
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function overrideIfApplicable($key, MenuBlockSettingsOverrideItem $overridden_settings, array &$configuration) {
    if ($overridden_settings->isApplicable($key)) {
      $configuration[$key] = $overridden_settings->get($key)->getCastedValue();
    }
  }

}
